#!/usr/bin/perl -w
#
# del.cgi
# John Simpson <jms1@jms1.net>
#
# Delete a rule
#
# $Id: del.cgi 106 2012-07-21 20:39:17Z jms1 $
#
###############################################################################
#
# Copyright (C) 2012 John M. Simpson.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 or version 3 of the
# license, at your option.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

require 5.003 ;
use strict ;
use CGI qw ( :standard ) ;

use Rules ;

###############################################################################
###############################################################################
###############################################################################

require_login() ;

########################################
# Check for cancel button.

if ( param ( 'cancel' ) )
{
	logit "cancelled\n" ;
	redirect_home() ;
}

########################################
# Read form data.

my $phase = ( param ( 'phase' ) || '' ) ;
my $seq   = ( param ( 'seq'   ) || '' ) ;
my $go    = ( param ( 'go'    ) || '' ) ;

unless ( $phase && $seq )
{
	logit "Either phase \"$phase\" or seq \"$seq\" is missing." ;
	redirect_home() ;
}

########################################
# make sure the user is authorized to change rules in this phase

my $query = "SELECT level FROM phases WHERE phase = $phase" ;
my $plevel = SQL_select ( $query ) ;
$plevel ||= 0 ;

if ( $access < $plevel )
{
	logit "ERROR: attempt to delete rule without proper access:"
		. " phase=[$phase] seq=[$seq] plevel=[$plevel]"
		. " access=[$access] userid=[$userid]\n" ;
	redirect_home() ;
}

########################################
# If we're confirming, do the deed.

if ( $go )
{
	########################################
	# delete the record

	$query = 'DELETE FROM rules WHERE '
		. SQL_quote ( $focus ) . ' LIKE recipient'
		. " AND phase = $phase"
		. " AND seq = $seq" ;
	SQL_do ( $query ) ;

	########################################
	# fix sequence numbers

	SQL_transaction() ;

	$query = 'SELECT id FROM rules'
		. ' WHERE ' . SQL_quote ( $focus ) . ' LIKE recipient'
		. " AND phase = $phase AND seq > $seq"
		. ' ORDER BY seq ASC' ;
	( my $id ) = SQL_selectfirst ( $query ) ;
	while ( $id )
	{
		my $q2 = "UPDATE rules SET seq = seq - 1 WHERE id = $id" ;
		SQL_do ( $q2 ) ;

		( $id ) = SQL_selectnext() ;
	}

	SQL_commit() ;

	########################################
	# done

	redirect_home() ;
}

########################################
# Not confirmed, we need to show the form.
# First read the existing rule into memory.

$query = 'SELECT recipient , type , sender , ip , rbl , delay , target'
	. ' , accept , description'
	. ' FROM rules WHERE'
	. SQL_quote ( $focus ) . ' LIKE recipient'
	. " AND phase = $phase"
	. " AND seq = $seq" ;
my @data = SQL_select ( $query ) ;

if ( $#data < 0 )
{
	logit "ERROR: attempt to delete non-existent rule: "
		. " phase=[$phase] seq=[$seq] userid=[$userid]\n" ;
	redirect_home() ;
}

my $r_recip  = ( shift @data || '' ) ;
my $r_type   = ( shift @data || '' ) ;
my $r_sender = ( shift @data || '' ) ;
my $r_ip     = ( shift @data || '' ) ;
my $r_rbl    = ( shift @data || '' ) ;
my $r_delay  = ( shift @data || '' ) ;
my $r_target = ( shift @data || '' ) ;
my $r_accept = ( shift @data || '' ) ;
my $r_desc   = ( shift @data || '' ) ;

########################################
# Figure data needed in order to show the form.

my $value = '' ;
if ( $r_type eq 'E' ) { $value = $r_sender ; }
if ( $r_type eq 'I' ) { $value = $r_ip     ; }
if ( $r_type eq 'R' ) { $value = $r_rbl    ; }
if ( $r_type eq 'G' ) { $value = $r_delay  ; }
if ( $r_type eq 'T' ) { $value = $r_target ; }

my $recip = $r_recip ;
if ( $recip eq '%' )
{
	$recip = 'All mailboxes on the system' ;
}
elsif ( $recip =~ m|^%\@(.+)| )
{
	$recip = "All mailboxes in the domain $1" ;
}

########################################
# Show the form.

print page_start ( "Confirm rule deletion" ) ;
print parse_template ( 'delete-rule.txt' ,
	'phase'	=> $phase ,
	'seq'	=> $seq ,
	'recip'	=> $recip ,
	'type'	=> ( $typename{$r_type} || $r_type ) ,
	'value'	=> $value ,
	'disp'	=> ( $r_accept ? 'ACCEPT' : 'REJECT' ) ,
	'desc'	=> ( $r_desc || '' ) ) ;
print page_end() ;
exit 0 ;
