#!/usr/bin/perl -w
#
# login.cgi
# John Simpson <jms1@jms1.net>
#
# Handle a login request.
#
# $Id: login.cgi 106 2012-07-21 20:39:17Z jms1 $
#
###############################################################################
#
# Copyright (C) 2012 John M. Simpson.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 or version 3 of the
# license, at your option.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################

require 5.003 ;
use strict ;
use CGI qw ( :standard ) ;

use Rules ;

###############################################################################
###############################################################################
###############################################################################
#
# Figure out what's going on.

init() ;

unless ( $dbconn )
{
	logit 'The database connection is not configured.' ;

	my $errbox = <<EOF ;
<p class='redbox'>The database connection is not configured.</p>
<br clear='all' />
EOF

	print page_start ( "Login Required" ) , $errbox , page_end() ;
	exit 0 ;
}

unless ( SQL_ping() )
{
	logit 'The database connection is not working.' ;

	my $errbox = <<EOF ;
<p class='redbox'>The database connection is not working.</p>
<br clear='all' />
EOF

	print page_start ( "Login Required" ) , $errbox , page_end() ;
	exit 0 ;
}

########################################
# If we already have access, redirect home.

$access && redirect_home() ;

########################################
# Read form input (if any.)

my $user = ( param ( "userid" ) || "" ) ;
my $pass = ( param ( "passwd" ) || "" ) ;

########################################
# If we have both fields, ask vpopmaild.

if ( $user && $pass )
{
	logit "asking vpopmaild about $user\n" ;
	$access = check_vpopmaild ( $user , $pass ) ;
}

########################################
# If vpopmail is cool with the ID/PW, create a session.
# Note that redirect_home() will create cookies containing $session and $focus.
# If the browser doesn't accept those cookies, they will end up here again.

if ( $access )
{
	$userid = $user ;
	$focus = $userid ;

	create_session() ;

	redirect_home() ;
}

########################################
# No good. Show the login form with an error message.

my $errbox = $login_err
	? "<p class='redbox'>$login_err</p><br clear='all'>"
	: "" ;

$session = '' ;
$userid = '' ;

print page_start ( "Login Required" ) ;
print parse_template ( "login_form.txt" ,
	"errbox" => $errbox ,
	"userid" => $userid ) ;
print page_end() ;
exit 0 ;
